@@ -27,9 +27,9 @@ extern "C" {
 #endif
 
 #define __ARB_VERSION 2
-#define __ARB_VERSION_MINOR 21
+#define __ARB_VERSION_MINOR 23
 #define __ARB_VERSION_PATCHLEVEL 0
-#define ARB_VERSION "2.21.0"
+#define ARB_VERSION "2.23.0"
 #define __ARB_RELEASE (__ARB_VERSION * 10000 + \
                          __ARB_VERSION_MINOR * 100 + \
                          __ARB_VERSION_PATCHLEVEL)
@@ -427,7 +427,9 @@ void arb_submul_fmpz(arb_t z, const arb_t x, const fmpz_t y, slong prec);
 
 void arb_fma(arb_t res, const arb_t x, const arb_t y, const arb_t z, slong prec);
 void arb_fma_arf(arb_t res, const arb_t x, const arf_t y, const arb_t z, slong prec);
+void arb_fma_si(arb_t res, const arb_t x, slong y, const arb_t z, slong prec);
 void arb_fma_ui(arb_t res, const arb_t x, ulong y, const arb_t z, slong prec);
+void arb_fma_fmpz(arb_t res, const arb_t x, const fmpz_t y, const arb_t z, slong prec);
 
 void arb_dot_simple(arb_t res, const arb_t initial, int subtract,
     arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
@@ -607,10 +609,15 @@ void arb_bell_ui(arb_t res, ulong n, slong prec);
 
 void arb_euler_number_fmpz(arb_t res, const fmpz_t n, slong prec);
 void arb_euler_number_ui(arb_t res, ulong n, slong prec);
+void arb_fmpz_euler_number_ui_multi_mod(fmpz_t res, ulong n, double alpha);
+void arb_fmpz_euler_number_ui(fmpz_t res, ulong n);
 
 void arb_partitions_fmpz(arb_t res, const fmpz_t n, slong prec);
 void arb_partitions_ui(arb_t res, ulong n, slong prec);
 
+void arb_primorial_nth_ui(arb_t res, ulong n, slong prec);
+void arb_primorial_ui(arb_t res, ulong n, slong prec);
+
 void arb_lambertw(arb_t res, const arb_t x, int flags, slong prec);
 
 ARB_INLINE void
@@ -874,6 +881,11 @@ _arb_atan_taylor_naive(mp_ptr y, mp_limb_t * error,
 void _arb_atan_taylor_rs(mp_ptr y, mp_limb_t * error,
     mp_srcptr x, mp_size_t xn, ulong N, int alternating);
 
+#define ARB_ATAN_NEWTON_PREC 3400
+
+void arb_atan_newton(arb_t res, const arb_t x, slong prec);
+void arb_atan_arf_newton(arb_t res, const arf_t x, slong prec);
+
 /* logarithm implementation */
 
 #define ARB_LOG_TAB11_BITS 7
@@ -890,7 +902,20 @@ ARB_DLL extern const mp_limb_t arb_log_tab11[1 << ARB_LOG_TAB11_BITS][ARB_LOG_TA
 ARB_DLL extern const mp_limb_t arb_log_tab12[1 << ARB_LOG_TAB12_BITS][ARB_LOG_TAB1_LIMBS];
 ARB_DLL extern const mp_limb_t arb_log_tab21[1 << ARB_LOG_TAB21_BITS][ARB_LOG_TAB2_LIMBS];
 ARB_DLL extern const mp_limb_t arb_log_tab22[1 << ARB_LOG_TAB22_BITS][ARB_LOG_TAB2_LIMBS];
-ARB_DLL extern const mp_limb_t arb_log_log2_tab[ARB_LOG_TAB2_LIMBS];
+ARB_DLL extern const mp_srcptr arb_log_log2_tab;
+
+void arb_log_newton(arb_t res, const arb_t x, slong prec);
+void arb_log_arf_newton(arb_t res, const arf_t x, slong prec);
+
+/* logarithms of primes */
+
+#define ARB_LOG_PRIME_CACHE_NUM 13
+
+ARB_DLL extern const mp_limb_t arb_log_p_tab[ARB_LOG_PRIME_CACHE_NUM][ARB_LOG_TAB2_LIMBS];
+void arb_log_primes_vec_bsplit(arb_ptr res, slong n, slong prec);
+
+void _arb_log_p_ensure_cached(slong prec);
+arb_srcptr _arb_log_p_cache_vec(void);
 
 /* exponential implementation */
 
@@ -932,6 +957,14 @@ void _arb_exp_sum_bs_powtab(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
 void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
     const fmpz_t x, flint_bitcnt_t r, slong N);
 
+#define ARB_LOG_REDUCTION_DEFAULT_MAX_PREC 4000000
+#define ARB_EXP_LOG_REDUCTION_PREC 2240
+#define ARB_LOG_NEWTON_PREC 2800
+
+void arb_exp_arf_log_reduction(arb_t res, const arf_t x, slong prec, int minus_one);
+void arb_exp_arf_generic(arb_t z, const arf_t x, slong prec, int minus_one);
+void arb_exp_arf(arb_t z, const arf_t x, slong prec, int minus_one, slong maglim);
+
 /* sin/cos implementation */
 
 /* only goes up to (pi/4) * 256 */
@@ -975,6 +1008,21 @@ void arb_sin_cos_wide(arb_t s, arb_t c, const arb_t x, slong prec);
 void _arb_sin_cos_generic(arb_t s, arb_t c, const arf_t x, const mag_t xrad, slong prec);
 void arb_sin_cos_generic(arb_t s, arb_t c, const arb_t x, slong prec);
 
+void arb_atan_gauss_primes_vec_bsplit(arb_ptr res, slong n, slong prec);
+
+#define ARB_ATAN_GAUSS_PRIME_CACHE_NUM 13
+
+#define ARB_SIN_COS_ATAN_REDUCTION_DEFAULT_MAX_PREC 4000000
+#define ARB_SIN_COS_ATAN_REDUCTION_PREC 2600
+
+ARB_DLL extern const mp_limb_t arb_atan_gauss_tab[ARB_ATAN_GAUSS_PRIME_CACHE_NUM][ARB_ATAN_TAB2_LIMBS];
+
+void _arb_atan_gauss_p_ensure_cached(slong prec);
+arb_srcptr _arb_atan_gauss_p_cache_vec(void);
+
+void arb_sin_cos_arf_atan_reduction(arb_t res1, arb_t res2, const arf_t x, slong prec);
+
+
 ARB_INLINE flint_bitcnt_t
 _arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
 {
@@ -1009,6 +1057,8 @@ void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
 
 void arb_atan_arf_bb(arb_t z, const arf_t x, slong prec);
 
+void arb_atan_frac_bsplit(arb_t s, const fmpz_t p, const fmpz_t q, int hyperbolic, slong prec);
+
 ARB_INLINE slong
 arb_allocated_bytes(const arb_t x)
 {
